#include <cstdlib>
#include <bgtu/computer_graphics_lab_work/utilities/default_renderer_linear.hpp>

namespace BGTU::ComputerGraphicsLabWork::Utilities {

	DefaultVoxelDrawerCache::ZElementAllocationBuffer::Cell *DefaultVoxelDrawerCache::ZElementAllocationBuffer::Page::try_allocate() {
		if (this->initialized < this->capacity) {
			Cell *cell = &(this->buffer[this->initialized++]);
			return cell;
		}
		return nullptr;
	}

	void DefaultVoxelDrawerCache::ZElementAllocationBuffer::free_elem(DefaultVoxelDrawerCache::ZElement *e) {
		Cell *cell = reinterpret_cast<Cell *>(e);
		cell->empty.next_empty = this->next_unallocated;
		this->next_unallocated = cell;
	}

	void DefaultVoxelDrawerCache::ZElementAllocationBuffer::release_resources() {
		Page *p;
		while (this->last_page != nullptr) {
			p = this->last_page;
			this->last_page = p->next;
			this->_native_free_page(p);
		}
		this->next_unallocated = nullptr;
	}

	void DefaultVoxelDrawerCache::ZElementAllocationBuffer::reset_allocations() {
		Page *p = this->last_page;
		while (p != nullptr) {
			p->reset_allocations();
			p = p->next;
		}
		this->next_unallocated = nullptr;
	}

	DefaultVoxelDrawerCache::ZElementAllocationBuffer::~ZElementAllocationBuffer() {
		this->release_resources();
	}

	DefaultVoxelDrawerCache::DefaultVoxelDrawerCache() : pixel_trace_elements_allocator{}, pixel_metadata_cache{.buffer = nullptr, .size = 0}, sorter{} {}

	DefaultVoxelDrawerCache::~DefaultVoxelDrawerCache() {
		if (this->pixel_metadata_cache.buffer != nullptr) {
			std::free(this->pixel_metadata_cache.buffer);
			this->pixel_metadata_cache.buffer = nullptr;
		}
		this->pixel_metadata_cache.size = 0;
	}

}